! This file is part of FD4: Four-Dimensional Distributed Dynamic Data structures
!  
! Copyright 2010-2012 Matthias Lieber, ZIH, TU Dresden, Germany
! matthias.lieber@tu-dresden.de
!  
! FD4 is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!  
! FD4 is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!  
! You should have received a copy of the GNU General Public License
! along with FD4.  If not, see <http://www.gnu.org/licenses/>.

!--------------------------------------------------------------!
!                                                              !
!                         Main FD4 Module                      !
!                                                              !
!--------------------------------------------------------------!

#include "fd4.h"

!! Main FD4 module which includes all interface types, constants, and routines of FD4.
!!
!! @author Matthias Lieber
module fd4_mod

  use kinds_mod

  use timing_mod

  !use fd4_mpi_mod, only:       MPI_I4K, MPI_I8K, MPI_I_K, MPI_R4K, MPI_R8K, MPI_R_K

  use fd4_globaldef_mod, only: FD4_X, FD4_Y, FD4_Z,    &
                               FD4_XY, FD4_XZ, FD4_YZ, &
                               FD4_XYZ, FD4_DIM
  
  use fd4_domain_mod, only:    fd4_domain,             &
                               fd4_domain_create,      &
                               fd4_domain_delete,      &
                               fd4_domain_clear,       &
                               fd4_domain_max_bext,    &
                               fd4_domain_dump_stats,  &
                               fd4_domain_init_metadata, &
                               fd4_domain_adapt_metadata
      
  use fd4_iter_mod, only:      fd4_iter,               &
                               fd4_iter_init,          &
                               fd4_iter_next,          &
                               fd4_iter_valid,         &
                               fd4_iter_offset,        &
                               fd4_iter_block_extent,  &
                               fd4_iter_get,           &
                               fd4_iter_put,           &
                               fd4_iter_get_ghost,     &
                               fd4_iter_get_facevar,   &
                               fd4_iter_put_ghost,     &
                               fd4_iter_empty,         &
                               fd4_iter_get_ghost_multi,&
                               fd4_iter_start_clock,   &
                               fd4_iter_stop_clock
  
  use fd4_vartab_mod, only:    fd4_vartab,             &
                               fd4_varinfo,            &
                               FD4_NOTHRES,            &
                               FD4_VARNAME_LEN,        &
                               FD4_CELLC,              &
                               FD4_FACEX,              &
                               FD4_FACEY,              &
                               FD4_FACEZ

  use fd4_util_mod, only:      fd4_util_put_array,     &
                               fd4_util_put_array_uv,  &
                               fd4_util_get_array,     &
                               fd4_util_get_value,     &
                               fd4_util_check_facevar, &
                               fd4_util_propagate_facevar, &
                               fd4_util_allocate_all_blocks

  use fd4_boundary_mod, only:  fd4_boundary_zerograd,  &
                               fd4_boundary_zerograd_block, &
                               fd4_boundary_spec,      &
                               fd4_boundary_spec_block

  use fd4_balance_mod, only:   fd4_balance_readjust,     &
                               FD4_BALANCE_HILBERT,      &
                               FD4_BALANCE_METIS,        &
                               FD4_BALANCE_SERIAL,       &
                               FD4_BALANCE_HSFC_SIMPLE,  &
                               FD4_BALANCE_HSFC_BINSRCH, &
                               FD4_BALANCE_HSFC_PARALLEL,&
                               FD4_BALANCE_HSFC2_PARALLEL,&
                               FD4_BALANCE_HSFC2_BINSRCH,&
                               FD4_BALANCE_HSFC_DISTR,   &
                               FD4_BALANCE_RCB,          &
                               fd4_balance_statistics,   &
                               fd4_balance_params

  use fd4_ghostcomm_mod, only: fd4_ghostcomm,          &
                               fd4_ghostcomm_exch,     &
                               fd4_ghostcomm_create,   &  
                               fd4_ghostcomm_delete

#ifdef FD4_VIS5D
  use fd4_vis5d_mod, only:     fd4_vis5d_open,         &
                               fd4_vis5d_write,        &
                               fd4_vis5d_close,        &
                               fd4_vis5d_attributes
#endif

#ifdef FD4_NETCDF4
  use fd4_netcdf4_mod, only:   fd4_netcdf4_open,       &
                               fd4_netcdf4_write,      &
                               fd4_netcdf4_close,      &
                               fd4_netcdf4_comm
#endif

  use fd4_couple_mod, only:    fd4_couple_put,         &
                               fd4_couple_get,         &
                               fd4_couple_mark_blocks, &
                               fd4_couple,             &
                               fd4_couple_create,      &
                               fd4_couple_add_array,   &
                               fd4_couple_add_partition, &
                               fd4_couple_add_var,     &
                               fd4_couple_set_local_3D_array, &
                               fd4_couple_set_local_4D_array, &
                               fd4_couple_commit,      &
                               fd4_couple_chg_local_3D_array, &
                               fd4_couple_chg_local_4D_array, &
                               fd4_couple_delete,      &
                               fd4_couple_debug_metadata, &
                               FD4_CPL_GET,            &
                               FD4_CPL_PUT,            &
                               FD4_CPL_BOTH
  
  use fd4_mpi_mod, only:       MPI_I4K, MPI_I8K, MPI_I_K, &
                               MPI_R4K, MPI_R8K, MPI_R_K, &
                               MPI_R_K_size
  

  use fd4_part_sfc_mod, only:  FD4_PART_SFC_MORTON,   &
                               FD4_PART_SFC_HILBERT,  &
                               FD4_PART_SFC_PARALLEL, &
                               FD4_PART_SFC_BINSRCH,  &
                               FD4_PART_SFC_SIMPLE

  use fd4_part_sfcd_mod, only: FD4_PART_SFCD_WGTX_MASTER,   &
                               FD4_PART_SFCD_WGTX_COLL

  public

end module fd4_mod
