! This file is part of FD4: Four-Dimensional Distributed Dynamic Data structures
!  
! Copyright 2010-2012 Matthias Lieber, ZIH, TU Dresden, Germany
! matthias.lieber@tu-dresden.de
!  
! FD4 is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!  
! FD4 is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!  
! You should have received a copy of the GNU General Public License
! along with FD4.  If not, see <http://www.gnu.org/licenses/>.

#ifndef _FD4_H_
#define _FD4_H_

! how to include the mpi interface?
#ifdef FD4_USE_MPI_MODULE
#define USE_MPI use mpi
#define INCLUDE_MPIFH
#else
#define USE_MPI
#define INCLUDE_MPIFH include 'mpif.h'
#endif

! error handling
! L0 =      no checks at all
! L1 =      output of basic and cheap error checks (default)
! L2 = L1 + some basic information
! L3 = L2 + expensive error checks (will hurt performance)
! L4 = L3 + verbose mode (tons of output)
! L5 = L4 + gigatons of output
!
! L1 error checks will be always carried through (due to semantics
! of subroutines), but output of these error messages will only be
! enabled if FD4_VERBOSE_LEVEL >= 1.
! L3 error checks are only carried through if FD4_VERBOSE_LEVEL >= 3.
!
! set default verbose level
#ifndef FD4_VERBOSE_LEVEL
#define FD4_VERBOSE_LEVEL 1
#endif
!
! L1 error checks
#if ( FD4_VERBOSE_LEVEL >= 1 )
#define FD4_L1_ASSERT(C,MSG) if(fd4_lerr(__LINE__,__FILE__,C,MSG))return
#define FD4_L1_ERROR(MSG)    call fd4_perr(__LINE__,__FILE__,MSG);return
#define FD4_L1_WARN(MSG)     call fd4_perr(__LINE__,__FILE__,MSG)
#else
#define FD4_L1_ASSERT(C,MSG) if(C)return
#define FD4_L1_ERROR(MSG)    return
#define FD4_L1_WARN(MSG)     continue
#endif
!
! L2 info
#if ( FD4_VERBOSE_LEVEL >= 2 )
#define FD4_L2_INFO(MSG) write(fd4_out,'(A,A)') FD4_LINEHD,MSG
#else
#define FD4_L2_INFO(MSG) continue
#endif
!
! L3 error checks
#if ( FD4_VERBOSE_LEVEL >= 3 )
#define FD4_L3_ASSERT(C,MSG) if(fd4_lerr(__LINE__,__FILE__,C,MSG))return
#else
#define FD4_L3_ASSERT(C,MSG) continue
#endif
!
! L4 info
#if ( FD4_VERBOSE_LEVEL >= 4 )
#define FD4_L4_INFO(MSG) write(fd4_out,'(A,A)') FD4_LINEHD,MSG
#else
#define FD4_L4_INFO(MSG) continue
#endif
!
! L5 info
#if ( FD4_VERBOSE_LEVEL >= 5 )
#define FD4_L5_INFO(MSG) write(fd4_out,'(A,A)') FD4_LINEHD,MSG
#else
#define FD4_L5_INFO(MSG) continue
#endif


! trim character string spaces
#define STRIM(x) x(1:len_trim(x))


#endif
! _FD4_H_
